<?php
if (!defined('PmWiki')) exit();

include_once("StaticPages.php");

# In StaticPages.php this is linked:  $LinkFunctions['<:page>'] = 'LinkToPossibleStaticPage';
function LinkToPossibleStaticPage($pagename,$imap,$path,$title,$txt,$fmt=NULL) {
    
    global $StaticPageDir, $ApplicationPathUrl;
	
    $link = LinkPage($pagename,$imap,$path,$title,$txt,$fmt);
    
    $targetPageName = getLinkTargetPageName($pagename, $path);
    
    $isStaticPage = isPossibleStaticPage($targetPageName);
    
    if (!$isStaticPage){
    	@removeStaticPage($targetPageName);
    	return $link;
    }
    
    $hrefToken = "'";
    $hrefStart = strpos($link, "'", strpos($link, "href"));    
    $hrefStop = strpos($link, "'", $hrefStart + 1);
    $hrefText = substr($link, $hrefStart + 1, $hrefStop - $hrefStart - 1);
    
    // Static pages will have only one argument in the form of "?n=Main.Page"
    $isStaticPage = FALSE;
    $hrefParts = explode("?", $hrefText);  // $args[0] is html link, $args[>0] is the arguments
    $wikiLinkPart = $hrefParts[1];
    
    if (count($hrefParts) <= 2 && (strcmp("n=", substr($wikiLinkPart, 0, 2)) == 0) ){
        $isStaticPage = TRUE;
    }
    
    if ($isStaticPage == FALSE){
        return $link;
    }
    
    $newLink = $ApplicationPathUrl . "/" . $StaticPageDir . "/"
        . getStaticPageNameFromWikiLink(substr($wikiLinkPart, 2));
        
    $link = substr_replace($link, $newLink, $hrefStart + 1, $hrefStop - $hrefStart - 1);
    
    return $link;
}


function getStaticPageNameFromWikiLink($wikiLink){
    # $wikiLink is in the form of "Main.HomePage"
    
    if (strlen($wikiLink) < 3 || strpos($wikiLink, ".") == FALSE){
    	return "Main_HomePage.html";  # Default if not specified
    }
    
    $wikiLinkGroupName = substr($wikiLink, 0, strpos($wikiLink, "."));
    $wikiLinkPageName = substr($wikiLink, strpos($wikiLink, ".") + 1, strlen($wikiLink) - 1);
    
    return $wikiLinkGroupName . "_" .$wikiLinkPageName .".html";
}

function createStaticPage($wikiLink){
   	
   	# $wikiLink is in the form of "Main.HomePage"
   	
   	$isStaticPage = isPossibleStaticPage($wikiLink);
   	
   	if (!$isStaticPage){
    	@removeStaticPage($wikiLink);
    	return FALSE;
    }
   	
    global $ScriptUrl, $StaticPageDir, $ApplicationPathDir;
    
    $wikiLinkAddress = $ScriptUrl . "?n=" . $wikiLink;
    $wikiLinkStaticPageDir = $ApplicationPathDir . "/" .
    	$StaticPageDir . "/" . getStaticPageNameFromWikiLink($wikiLink);
	
    if (! ($wikiLinkPageContents = file_get_contents($wikiLinkAddress)) ){
    	echo "Could not open URL for copying:  " .  $wikiLinkAddress;
    	return FALSE;
    }
    
	
	ensureStaticPageDirExists();
	    
    if (!($fpStaticPage = fopen($wikiLinkStaticPageDir , "w+"))){
       echo "Could not open file for writing:  " .  $wikiLinkStaticPageDir;
       return FALSE;
    }
    
    if (!(fwrite($fpStaticPage, $wikiLinkPageContents))){
       echo "Could not write to file:  " .  $wikiLinkStaticPageDir;
       return FALSE;
    }
    
    fclose($fpStaticPage);
    
    return TRUE;
    
}

function removeStaticPage($wikiLink){
	global $StaticPageDir, $ApplicationPathDir;
	
	if (isset($wikiLink) && strlen($wikiLink) > 1 && !(substr($wikiLink, 0, 1) == ".")){
		return unlink($ApplicationPathDir . "/" . 
			$StaticPageDir . "/" . getStaticPageNameFromWikiLink($wikiLink));  
	}
	
	return FALSE;
}


function UpdateStaticPage($pagename,&$page,&$new){
	global $EnablePost;
	
	# Will get called when the edit page opens, and then again on an edit page action
	if ($EnablePost){
		createStaticPage($pagename);
	}
}


function HandleStaticPageGenerator($pagename, $auth = 'admin') {
	
	global $ScriptUrl, $ApplicationPathDir, $WorkDir, $StaticPageDir;
	global $StaticPageGeneratorActionName;
	
	chdir("$ApplicationPathDir/cookbook");
	include("StaticPageGenerator.php");
}


function isPossibleStaticPage($pagename){
		
	# $pagename in the form of "Main.HomePage"
	
	# Standard Text match Exclude - Note to deprecate
    global $StaticPageExclude;
    if (isset($StaticPageExclude) && is_array($StaticPageExclude) 
    		&& in_array($pagename, $StaticPageExclude)){
    	// Remove Static Page if it exists
    	return FALSE;
    }
    
    # Regular Expression Exclude
    global $StaticPageRegularExpressionExclude;
    if (isset($StaticPageRegularExpressionExclude) && is_array($StaticPageRegularExpressionExclude)){
    	for ($index = 0; $index < sizeof($StaticPageRegularExpressionExclude); $index++) {
			$currentValue = $StaticPageRegularExpressionExclude[$index];
			if (preg_match($StaticPageRegularExpressionExclude[$index], $pagename) > 0){
				return FALSE;
			}
		}    	
    }
    
    #If file does not exist in $WorkDir directory (eg wiki.d) then will not be a static file
    global $ApplicationPathDir, $WorkDir;
    $linkedWorkingPageDir = $ApplicationPathDir . "/" .
    	$WorkDir . "/" . $pagename;
    #echo "linkedWorkingPageDir:  $linkedWorkingPageDir <br/>";
    if (!file_exists($linkedWorkingPageDir)){
    	return FALSE;
    }
    
    # Not part of the exclude list
    return TRUE;
    
}

function ensureStaticPageDirExists(){
	
	global $StaticPageDir;
	if (!is_dir($StaticPageDir)){
    	
    	# Permissions - Global Read and Write Access, No Execute
    	if (!MakeDirectoryPath($StaticPageDir, 0006)){
    		echo "Could not create output directory for static files:  " .  $StaticPageDir;
       		exit;
    	}
    }
}

function MakeDirectoryPath($path, $mode = 0755){
	
	# Attempt to create $dir directory
	if (is_dir($path) || @mkdir($path,$mode)){
		return TRUE;
	}
	# Attempt to create parent directory
	if (!MakeDirectoryPath(dirname($path),$mode)){
		return FALSE;
	}
	
	# Attempt again to create this directory
	return @mkdir($path,$mode);
}

function getLinkTargetPageName($pagename, $path){
	global $QueryFragPattern;
    
    if (!preg_match("/^\\s*([^#?]+)($QueryFragPattern)?$/", $path, $match)){
    	return "";	
    }
    
    $targetPageName = MakePageName($pagename, $match[1]); 
    if (!$targetPageName){
    	return "";
    }else{
    	return $targetPageName;
    }
}
?>